#if defined _bossengine_included
    #endinput
#endif
#define _bossengine_included

#define INVALID_EGG     view_as<Egg>(-1)
#define INVALID_MONSTER view_as<Monster>(-1)

// Eggs are monster prefabs. Use them to create monsters
//
methodmap Egg
{
    // Destroys an egg
    // 
    // Note: Only the plugin owning the egg may destroy it (but it will be automatically
    // destroyed if the plugin owning it is unloaded)
    // 
    public native void Destroy();
    
    // Retrieves the name of an egg
    //
    // @param buffer        Return buffer
    // @param maxlength     Maxlength of the buffer
    // 
    public native void GetName(char[] buffer, int maxlength);
    
    // Looks for attribute data of a given name
    // 
    // @param name          String used to indentify data
    // @return              True if data exists, false otherwise
    // @error               Invalid egg
    // 
    public native bool HasAttribute(const char[] name);
    
    // Gets attribute string data
    // 
    // @param name          String used to indentify an attribute
    // @param buffer        Buffer to return the data
    // @return              True if data was found and retrieved, false otherwise
    // @error               Invalid egg
    //
    public native bool GetAttribute(const char[] name, char[] buffer, int maxlength);
    
    // Adds attribute data (will be used when the eggs hatches into a monster)
    // 
    // Note: If an attribute already exists, the data will be overwritten
    // 
    // @param name          String used to indentify an attribute
    // @param data          String value
    // @error               Invalid egg
    // 
    public native void SetAttribute(const char[] name, const char[] data = "");
    
    // Removes attribute data
    // 
    // @param name          String used to indentify an attribute
    // @error               Invalid egg
    //
    public native void ClearAttribute(const char[] name);
    
    // Removes ALL attribute data
    // 
    // @error               Invalid egg
    //
    public native void DumpAttributes();
    
    // Loads additional attribute date from a config
    //
    // @param filename      Filename of the config
    // @return              True if config was loaded, false if not
    //
    public native bool LoadConfig(const char[] filename);
}



methodmap Monster
{
    // Client that the monster is bound to
    // 
    property int Client
    {
        public native get();
    }
    
    // Egg index of the egg used in creation
    // 
    property int EggID
    {
        public native get();
    }
    
    // Retrieves the name of a monster
    //
    // @param buffer        Return buffer
    // @param maxlength     Maxlength of the buffer
    // 
    public native void GetName(char[] buffer, int maxlength);
    
    // Destroys a monster
    // 
    public native void Destroy();
    
    // Looks for property data of a given name
    // 
    // @param name          String used to indentify data
    // @return              True if data exists, false otherwise
    // @error               Invalid monster
    // 
    public native bool HasProperty(const char[] name);
    
    // Add property data. Such data can be modified by any plugin with a valid monster ID
    // 
    // @param name          String used to indentify an attribute
    // @param buffer        String data to store
    // @error               Invalid monster
    // @return              True if data was able to be set, false otherwise
    //
    public native bool SetProperty(const char[] name, const char[] buffer = "");
    
    // Gets property data
    // 
    // @param name          String used to indentify an attribute
    // @param buffer        Buffer to return the data
    // @return              True if data was found and retrieved, false otherwise
    // @error               Invalid monster
    // 
    public native bool GetProperty(const char[] name, char[] buffer, int maxlength);
    
    // Deletes property data. No trace will be left
    // 
    // @param name          String used to indentify data
    // @error               Invalid monster
    // @return              True if property was removed, false otherwise
    // 
    public native bool ClearProperty(const char[] name);
    
    // Looks for attribute data of a given name
    // 
    // @param name          String used to indentify data
    // @return              True if data exists, false otherwise
    // @error               Invalid monster
    // 
    public native bool HasAttribute(const char[] name);
    
    // Gets attribute string data
    // 
    // @param name          String used to indentify an attribute
    // @param buffer        Buffer to return the data
    // @return              True if data was found and retrieved, false otherwise
    // @error               Invalid monster
    //
    public native bool GetAttribute(const char[] name, char[] buffer, int maxlength);
}



// Creates a new egg
// 
// @param name          Unique string to name the egg
// @return              A new Egg id, or INVALID_EGG if an egg already exists under that name
// 
native Egg BossEngine_CreateEgg(const char[] name);

// Quick check to see if an egg exists (prevents errors)
// 
// @param item          ID of the egg to check
// @return              True if the egg exists, false otherwise
//
native bool BossEngine_IsValidEgg(Egg item);

// Attempts to find an egg id by name
//
// @param name          The name to search for
// @return              An egg ID if found, or INVALID_EGG if not
//
native Egg BossEngine_FindEggByName(const char[] name);

// Creates a new monster from an egg
// 
// @param client        Index of the unlucky client
// @param item          Egg to load attributes from
// @return              A new Monster id, or INVALID_MONSTER if failure occurs
// 
native Monster BossEngine_CreateMonster(int client, Egg item);

// Quick check to see if an monster exists (prevents errors)
// 
// @param thing         ID of the monster to check
// @return              True if the monster exists, false otherwise
//
native bool BossEngine_IsValidMonster(Monster thing);

// Gets the ID of the monster assigned to a given client
// 
// @param client        Index of the client
// @return              A monster ID if the client has one, INVALID_MONSTER if not
// @error               Invalid client index, or client not ingame
//
native Monster BossEngine_GetClientMonster(int client);



// Called when an egg is created
//
// @param item          The ID of the egg being created
// @param name          Name of the egg
//
forward void BossEngine_EggCreated(Egg item, const char[] name);

// Called when an egg is destroyed
//
// @param item          The ID of the egg being destroyed
// @param name          Name of the egg
//
forward void BossEngine_EggDestroyed(Egg item, const char[] name);

// Called when a monster is created
//
// @param thing         The ID of the monster being created
// @param name          Name of the monster being created
// @param client        Index of the unlucky client
//
forward void BossEngine_MonsterCreated(Monster thing, const char[] name, int client);

// Called when a monster is destroyed
// Note: attribute data is still available at this time
//
// @param thing         The ID of the monster being destroyed
// @param name          Name of the monster being created
// @param client        Index of the unlucky client
//
forward void BossEngine_MonsterDestroyed(Monster thing, const char[] name, int client);



//----------------------------//
// DO NOT EDIT ANYTHING BELOW //
//----------------------------//

// Heartbeat function
// Note: when eggs are created, a private forward is also created using this callback.
// It is never actually executed, it is simply here to verify that the plugin owning an
// object is still loaded
//
public void BossEngine_Heartbeat()
{
    return;
}

public SharedPlugin:__pl_bossengine = 
{
    name = "bossengine",
    file = "bossengine.smx",
    required = 1,
};
